/* DO NOT EDIT THIS FILE - it is machine generated */

#ifndef __java_lang_VMDouble__
#define __java_lang_VMDouble__

#include <jni.h>

#ifdef __cplusplus
extern "C"
{
#endif

JNIEXPORT jlong JNICALL Java_java_lang_VMDouble_doubleToLongBits (JNIEnv *env, jclass, jdouble);
JNIEXPORT jlong JNICALL Java_java_lang_VMDouble_doubleToRawLongBits (JNIEnv *env, jclass, jdouble);
JNIEXPORT jdouble JNICALL Java_java_lang_VMDouble_longBitsToDouble (JNIEnv *env, jclass, jlong);

#ifdef __cplusplus
}
#endif

#endif /* __java_lang_VMDouble__ */
