/* DO NOT EDIT THIS FILE - it is machine generated */

#ifndef __java_io_ObjectInputStream__
#define __java_io_ObjectInputStream__

#include <jni.h>

#ifdef __cplusplus
extern "C"
{
#endif

JNIEXPORT jobject JNICALL Java_java_io_ObjectInputStream_currentClassLoader (JNIEnv *env, jclass, jobject);
JNIEXPORT jobject JNICALL Java_java_io_ObjectInputStream_allocateObject (JNIEnv *env, jobject, jclass);
JNIEXPORT void JNICALL Java_java_io_ObjectInputStream_callConstructor (JNIEnv *env, jobject, jclass, jobject);

#ifdef __cplusplus
}
#endif

#endif /* __java_io_ObjectInputStream__ */
