/* DO NOT EDIT THIS FILE - it is machine generated */

#ifndef __gnu_java_awt_peer_gtk_GdkFontMetrics__
#define __gnu_java_awt_peer_gtk_GdkFontMetrics__

#include <jni.h>

#ifdef __cplusplus
extern "C"
{
#endif

JNIEXPORT jintArray JNICALL Java_gnu_java_awt_peer_gtk_GdkFontMetrics_initState (JNIEnv *env, jobject, jstring, jint, jint);
JNIEXPORT jint JNICALL Java_gnu_java_awt_peer_gtk_GdkFontMetrics_stringWidth (JNIEnv *env, jobject, jstring, jint, jint, jstring);

#ifdef __cplusplus
}
#endif

#endif /* __gnu_java_awt_peer_gtk_GdkFontMetrics__ */
