/* DO NOT EDIT THIS FILE - it is machine generated */

#ifndef __java_lang_Double__
#define __java_lang_Double__

#include <jni.h>

#ifdef __cplusplus
extern "C"
{
#endif

JNIEXPORT jdouble JNICALL Java_java_lang_Double_parseDouble (JNIEnv *env, jclass, jstring);
JNIEXPORT jstring JNICALL Java_java_lang_Double_toString (JNIEnv *env, jclass, jdouble, jboolean);
JNIEXPORT void JNICALL Java_java_lang_Double_initIDs (JNIEnv *env, jclass);

#ifdef __cplusplus
}
#endif

#endif /* __java_lang_Double__ */
