/* DO NOT EDIT THIS FILE - it is machine generated */

#ifndef __gnu_java_awt_peer_gtk_GThreadNativeMethodRunner__
#define __gnu_java_awt_peer_gtk_GThreadNativeMethodRunner__

#include <jni.h>

#ifdef __cplusplus
extern "C"
{
#endif

JNIEXPORT void JNICALL Java_gnu_java_awt_peer_gtk_GThreadNativeMethodRunner_nativeRun (JNIEnv *env, jobject, jlong, jlong);

#ifdef __cplusplus
}
#endif

#endif /* __gnu_java_awt_peer_gtk_GThreadNativeMethodRunner__ */
