/* DO NOT EDIT THIS FILE - it is machine generated */

#ifndef __java_nio_channels_Channels__
#define __java_nio_channels_Channels__

#include <jni.h>

#ifdef __cplusplus
extern "C"
{
#endif

JNIEXPORT jobject JNICALL Java_java_nio_channels_Channels_newInputStream (JNIEnv *env, jclass, jobject);
JNIEXPORT jobject JNICALL Java_java_nio_channels_Channels_newOutputStream (JNIEnv *env, jclass, jobject);

#ifdef __cplusplus
}
#endif

#endif /* __java_nio_channels_Channels__ */
